/*
 * Expander.c
 *
 * PCF8574(i2c) pipojen k ATmega8
 *
 * Created: 13.2.2013 15:53:23
 *  Author: Standa
 */ 

#include <avr/io.h>
#define F_CPU 8000000UL
#include "twi_lib.h"					//knihovna i2c
#include <util/delay.h>
#include <stdio.h>

#define PCF8574	0b01000000				//i2c adresa obvodu
#define PCF8574A 0b01110000				//i2c adresa obvodu

#define EN_HIGH 0b10000000
#define EN_LOW 0b01111111

#define RS_HIGH 0b00010000
#define RS_LOW 0b11101111

unsigned char lcd_data;

//Definice pin pro LCD:
/*
#define LCD_RS 4
#define LCD_E  7

#define LCD_D4 0
#define LCD_D5 1
#define LCD_D6 2
#define LCD_D7 3 
*/

#define HOME 0x02
#define CLR 0x01
/*
void Lcd_Init();
void Lcd_Char(unsigned char chr);
void Lcd_Cmd(unsigned char cmd);
void Lcd_Out(char *str);
void Lcd_Gotoxy(unsigned char radek, unsigned char sloupec);
void Write_PIO(unsigned char data);
*/

//Zpis do PIO
void Write_PIO(unsigned char data)
{
	i2c_start(PCF8574+I2C_WRITE);
	i2c_write(data);
	i2c_stop();
}
	
//Command
void Lcd_Cmd(unsigned char cmd)
{	
	lcd_data = ((cmd & 0xF0)>>4);
	lcd_data &= RS_LOW;
	lcd_data |= EN_HIGH;
	Write_PIO(lcd_data);
	_delay_us(5);
	lcd_data &= EN_LOW;
	Write_PIO(lcd_data);
	_delay_us(100);
	
	lcd_data = (cmd & 0x0F);
	lcd_data &= RS_LOW;
	lcd_data |= EN_HIGH;
	Write_PIO(lcd_data);
	_delay_us(5);
	lcd_data &= EN_LOW;
	Write_PIO(lcd_data);
	
//Pokud je vsledek v podmnce nula, jedn se o 
//pkaz "Vyma dislej" nebo "Nvrat na zatek", kter
//vyaduj del interval

	if(cmd & 0b11111100) _delay_us(100);else _delay_ms(2);
} 

//Char
void Lcd_Chr(unsigned char chr)
{
	lcd_data = ((chr & 0xF0)>>4);
	lcd_data |= (RS_HIGH | EN_HIGH);
	Write_PIO(lcd_data);
	_delay_us(5);
	lcd_data &= EN_LOW;
	Write_PIO(lcd_data);
	_delay_us(100);
	
	lcd_data = (chr & 0x0F);
	lcd_data |= (RS_HIGH | EN_HIGH);
	Write_PIO(lcd_data);
	_delay_us(5);
	lcd_data &= EN_LOW;
	Write_PIO(lcd_data);
}

//inicializace LCD
void Lcd_Init()
{
	Write_PIO(0b00000000);
	_delay_ms(40);							//ekej 40ms - pro typ LCD AMC2004AR-B
	
//zapi 3 x 0x03h, 8bitov komunikace
	lcd_data = 0b00000011;
	Write_PIO(lcd_data);
	lcd_data |= EN_HIGH;					//EN = 1
	Write_PIO(lcd_data);
	_delay_us(5);
	lcd_data &= EN_LOW;						//EN = 0
	Write_PIO(lcd_data);			      
	_delay_ms(5);							//ekej > 4,1ms
	
	lcd_data |= EN_HIGH;					//EN = 1
	Write_PIO(lcd_data);
	_delay_us(5);
	lcd_data &= EN_LOW;						//EN = 0
	Write_PIO(lcd_data);	
	_delay_us(100);
	
	lcd_data |= EN_HIGH;					//EN = 1
	Write_PIO(lcd_data);
	_delay_us(10);
	lcd_data &= EN_LOW;						//EN = 0
	Write_PIO(lcd_data);	
	_delay_us(100);
 
//4bitov komunikace 0x02h
	lcd_data = 0b00000010;
	Write_PIO(lcd_data);
	lcd_data |= EN_HIGH;					//EN = 1
	Write_PIO(lcd_data);
	_delay_us(5);
	lcd_data &= EN_LOW;						//EN = 0
	Write_PIO(lcd_data);			      
	_delay_us(100);	            
	
	Lcd_Cmd(0b00101000);		  //dva dky
	Lcd_Cmd(0b00000001);		  //sma LCD
	Lcd_Cmd(0b00001100);		  //zapni LCD
	Lcd_Cmd(0b00000110);		  //incrementace
}

//Zatek vpisu na souadnice x,y
void Lcd_Gotoxy(unsigned char radek, unsigned char sloupec)
{
	switch(radek)
	{
		case 1:radek=0x00;break;
		case 2:radek=0x40;break;
		case 3:radek=0x14;break;
		case 4:radek=0x54;break;
	}	
 
	Lcd_Cmd(0b10000000 | radek | sloupec);
}

//Vpis textu
void Lcd_Out(char *str)
{
	unsigned char x = 0;
	while(str[x]!=0)
	{
		Lcd_Chr(str[x]);
		x++;
	}
}

int lcd_putchar(char c, FILE *stream)
{
	Lcd_Chr(c);
	return 0;
}

static FILE mystdout = FDEV_SETUP_STREAM(lcd_putchar,NULL,_FDEV_SETUP_WRITE);

int main(void)
{
	stdout = &mystdout;
	i2c_init();
	
	Lcd_Init();
	
	printf("LCD 4x20znak");
	Lcd_Gotoxy(2,0);
	printf("i2c PIO PCF8574");
	Lcd_Gotoxy(3,0);
	printf("ATmega8,F_CPU = 8MHz");
	Lcd_Gotoxy(4,0);
	printf("LINE 4");
	
    while(1)
    {
        //TODO:: Please write your application code 
    }
}